#ifndef GAME_H
#define GAME_H

class Window;

#ifndef PUBLIC_HEADER
#include "Window.h"
#endif /* PUBLIC_HEADER */

#include "Input/KeyboardState.h"
#include "Input/MouseState.h"
#include "Graphics/D3D11_Core.h"
#include "Graphics/ContentManager.h"

/// <summary>
/// Base class for Game objects that provide the basic framework for a game
/// (i.e. takes care of managing the window, handles time delays for the update
/// cycle, etc)
/// </summary>
class Game
{
  public:
    /// <summary>
    /// Creates the game instance, including the window that will be used for
    /// it
    /// </summary>
    /// <remarks>
    /// A default 640x480 overlapped window is created.  It can be resized or
    /// made full screen by calling the appropriate functions on the instance.
    /// </remarks>
    Game(WCHAR* title);
    
    /// <summary>
    /// Performs any needed cleanup
    /// </summary>
    virtual ~Game();
    
    /// <summary>
    /// Runs the game until it is exited for any reason
    /// </summary>
    void Run();
    
    /// <summary>
    /// Resizes the window for the game
    /// </summary>
    /// <remarks>
    /// This function tells the instance to resize its window.  Any subclass
    /// operations for when a window is resized (either due to this function or
    /// the user dragging the window border) should be placed in an override of
    /// OnResize.
    /// </remarks>
    /// <param name="width">
    /// width of the new client area, in pixels 
    /// </param>
    /// <param name="height">
    /// height of the new client area, in pixels
    /// </param>
    void Resize(UINT width,UINT height);
    
    /// <summary>
    /// Sets whether the game's window should be full screen or not
    /// <remarks>
    /// This tries to preserve the current window's client area dimensions.  If
    /// a different resolution is desired, call Resize first with the updated
    /// resolution.
    /// </remarks>
    /// <summary>
    /// <param name="enable">
    /// true  if the window should be fullscreen
    /// false if the window should be in windowed mode
    /// </param>
    void Fullscreen(bool enable);
    
    /// <summary>
    /// Retrieves the desktop's dimensions in pixels
    /// <summary>
    /// <param name="width">
    /// where to put the desktop width
    /// </param>
    /// <param name="height">
    /// where to put the desktop height
    /// </param>
    /// <returns>
    /// true  if the desktop size was determined
    /// false otherwise
    /// </returns>
    bool GetDesktopSize(UINT& width,UINT& height) const;
    
  protected:
    /// <summary>
    /// Causes the instance to exit its run loop at the end of the current
    /// iteration
    /// </summary>
    void Exit();
    
    /// <summary>
    /// Retrieves the dimensions of the window's client area
    /// </summary>
    /// <param name="width">
    /// where to put the retrieved width
    /// </param>
    /// <param name="height">
    /// where to put the retrieved height
    /// </param>
    void GetWindowClientDim(UINT& width,UINT& height) const;
    
    /// <summary>
    /// Retrieves the current mouse state
    /// </summary>
    /// <returns>
    /// current mouse state
    /// </returns>
    const MouseState& GetMouseState() const;
    
    /// <summary>
    /// Retrieves the current keyboard state
    /// </summary>
    /// <returns>
    /// current keyboard state
    /// </returns>
    const KeyboardState& GetKeyboardState() const;
    
    /// <summary>
    /// Where subclasses should load their content (graphical or otherwise)
    /// </summary>
    /// <param name="content">
    /// content manager to use for loading resources
    /// </param>
    virtual void LoadContent() = 0;
    
    /// <summary>
    /// Where subclasses should unload their content (graphical or otherwise)
    /// just before deallocation of the instance
    /// </summary>
    /// <param name="content">
    /// content manager to use for unloading resources
    /// </param>
    virtual void UnloadContent() = 0;
    
    /// <summary>
    /// Where subclasses should update their game state
    /// </summary>
    /// <param name="ms">
    /// number of milliseconds since the update
    /// </param>
    virtual void Update(UINT ms) = 0;
    
    /// <summary>
    /// Where subclasses should draw the current frame
    /// </summary>
    /// <param name="ms">
    /// number of milliseconds since the last frame
    /// </param>
    virtual void Draw(UINT ms) = 0;
    
    /// <summary>
    /// Handler for when the window is resized
    /// </summary>
    /// <remarks>
    /// If overridden by a subclass, the base class version must be called to
    /// ensure that base class members are properly updated.
    /// </remarks>
    /// <remarks>
    /// It is strongly recommended for subclasses to override this function at
    /// least to update the projection matrix of any cameras, and handle
    /// resizing of the viewport(s).  The next Draw call will also be against
    /// a pipeline in the same state (pipeline state, viewports, etc) as the
    /// first Draw call (aside from things being resized).
    /// </remarks>
    /// <param name="width">
    /// width of the new client area, in pixels 
    /// </param>
    /// <param name="height">
    /// height of the new client area, in pixels
    /// </param>
    virtual void OnResize(UINT width,UINT height);
    
    /// <summary>
    /// Retrieves the graphics interface
    /// </summary>
    /// <returns>
    /// reference to the graphics interface
    /// </returns>
    D3D11_Core& GetGraphics();
    
    /// <summary>
    /// Retrieves the content manager
    /// <remarks>
    /// should only be called between and during LoadContent/UnloadContent
    /// </remarks>
    /// </summary>
    /// <returns>
    /// reference to the content manager
    /// </returns>
    ContentManager& GetContentManager();
    
    /// <summary>
    /// Retrieves the number of milliseconds per frame
    /// </summary>
    /// <returns>
    /// number of milliseconds per frame
    /// </returns>
    UINT GetMSPerFrame() const;
    
    /// <summary>
    /// Sets the number of milliseconds per frame
    /// </summary>
    /// <param name="mspf">
    /// number of milliseconds per frame
    /// </returns>
    void SetMSPerFrame(UINT mspf);
    
  private:
    // disabled
    Game(const Game& cpy);
    Game& operator=(const Game& cpy);
    
    /// <summary>
    /// Call back function to provide instances of Window for handling resize
    /// events
    /// </summary>
    /// <param name="arg">
    /// pointer to the Game instance that owns the window that was resized
    /// </param>
    /// <param name="width">
    /// height of the new window client area, in pixels
    /// </param>
    /// <param name="height">
    /// width of the new window client area, in pixels
    /// </param>
    static void ResizeHandler(void* arg,UINT width,UINT height);
    
    /// <summary>
    /// Performs operations to draw the current frame
    /// </summary>
    /// <param name="ms">
    /// number of milliseconds since the last frame
    /// </param>
    void Render(UINT ms);
    
    /// <summary>
    /// window for the game
    /// </summary>
    Window* m_window;
    
    /// <summary>
    /// whether the Run loop should continue to occur (true) or not (false)
    /// </summary>
    bool m_run;
    
    /// <summary>
    /// D3D11 core for m_window
    /// </summary>
    D3D11_Core* m_graphics;
    
    /// <summary>
    /// content manager for loading/unloading graphics resources
    /// </summary>
    ContentManager* m_content;
    
    /// <summary>
    /// number of milliseconds to wait between frames
    /// </summary>
    /// <remarks>
    /// constructor defaults this to 60 fps
    /// </remarks>
    UINT m_mspf;
};

#endif /* GAME_H */
