#ifndef VERTEXBUFFER_H
#define VERTEXBUFFER_H

#include "Graphics/D3D11_Core.h"

class VertexBufferArray;

/// <summary>
/// Base class for vertex buffers
/// </summary>
class VertexBuffer
{
  public:
    virtual ~VertexBuffer();
    
    /// <summary>
    /// Setup the vertex buffer for usage with the specified context
    /// <summary>
    /// <param name="context">
    /// D3D11 context to apply the buffer to
    /// </param>
    void MakeActive(ID3D11DeviceContext* context);
    
    /// <summary>
    /// Retreives the number of entries in the buffer
    /// <summary>
    /// <returns>
    /// number of entries in the buffer
    /// </returns>
    UINT GetNumEntries() const;
    
  protected:
    VertexBuffer(ID3D11Buffer* buffer,UINT num);
    
    /// <summary>
    /// Creates a vertex buffer
    /// <remarks>
    /// subclasses should check the 
    /// </remarks>
    /// </summary>
    /// <param name="graphics">
    /// D3D11 instance to create the vertex buffer for
    /// </param>
    /// <param name="num">
    /// number of entries in the buffer
    /// </param>
    /// <param name="size">
    /// number of bytes in 1 entry
    /// </param>
    /// <param name="host_write">
    /// true  if the CPU is expected to frequently updated the buffer
    /// false otherwise
    /// </param>
    /// <param name="data">
    /// initial data for the vertex buffer
    /// </param>
    static ID3D11Buffer* CreateBuffer(D3D11_Core& graphics,UINT num,UINT size,
      bool host_write,const void* data);
    
    /// <summary>
    /// Retrieves the stride between elements
    /// </summary>
    /// <returns>
    /// number of bytes between each element in the vertex buffer
    /// </returns>
    virtual UINT GetStride() const = 0;
    
    /// <summary>
    /// Retrieve the vertex buffer
    /// </summary>
    /// <returns>
    /// pointer to the vertex buffer
    /// </returns>
    ID3D11Buffer* GetBuffer();
    
  private:
    // disabled
    VertexBuffer();
    VertexBuffer(const VertexBuffer& cpy);
    VertexBuffer& operator=(const VertexBuffer& cpy);
    
    /// <summary>
    /// number of entries in the buffer
    /// </summary>
    UINT m_num;
    
    /// <summary>
    /// D3D11 vertex buffer
    /// </summary>
    ID3D11Buffer* m_buffer;
    
    // allow VertexBufferArray full access to the members since it is an
    // optimization of VertexBuffer for multiple buffers
    friend VertexBufferArray;
};

#endif /* VERTEXBUFFER_H */
