#ifndef VERTEXBUFFERARRAY_H
#define VERTEXBUFFERARRAY_H

#include <d3d11.h>
#include "Graphics/Buffers/VertexBuffer.h"

/// <summary>
/// Container of multiple vertex buffers and allows for quickly sending
/// multiple vertex buffers to a D3D11 context
/// </summary>
class VertexBufferArray
{
  public:
    /// <summary>
    /// Creates a vertex buffer array with the specified number of elements.
    /// The array is initialized to all elements clear of a vertex buffer.
    /// </summary>
    /// <param name="num_buffers">
    /// number of buffers that will be in the array
    /// </param>
    VertexBufferArray(UINT num_buffers);
    
    ~VertexBufferArray();
    
    /// <summary>
    /// Sets the element at that specified index to the specified buffer
    /// <summary>
    /// <remarks>
    /// If a vertex buffer is to be destroyed, then it must be cleared from
    /// this before the array is used again
    /// </remarks>
    /// <param name="index">
    /// where in the array to set the buffer
    /// <remarks>
    /// This does not perform bounds checking
    /// </remarks>
    /// </param>
    /// <param name="buffer">
    /// buffer to set the element to
    /// </param>
    void Set(UINT index,const VertexBuffer& buffer);
    
    /// <summary>
    /// Removes any vertex buffer from the specified index
    /// </summary>
    /// <param name="index">
    /// which index to clear
    /// <remarks>
    /// This does not perform bounds checking
    /// </remarks>
    /// </param>
    void Clear(UINT index);
    
    /// <summary>
    /// Setup the vertex buffers for usage with the specified context
    /// <summary>
    /// <param name="context">
    /// D3D11 context to apply the buffers to
    /// </param>
    void MakeActive(ID3D11DeviceContext* context);
    
  private:
    // disabled
    VertexBufferArray();
    VertexBufferArray(const VertexBufferArray& cpy);
    VertexBufferArray& operator=(const VertexBufferArray& cpy);
    
    /// <summary>
    /// D3D11 vertex buffers
    /// </summary>
    ID3D11Buffer** m_buffer;
    
    /// <summary>
    /// array for the stide of each vertex buffer
    /// </summary>
    UINT* m_strides;
    
    /// <summary>
    /// array for offsets into the vertex buffers
    /// </summary>
    UINT* m_offsets;
    
    /// <summary>
    /// number of elements in the various parallel arrays
    /// </summary>
    UINT m_num;
};

#endif /* VERTEXBUFFERARRAY_H */
