#ifndef CONTENTMANAGER_H
#define CONTENTMANAGER_H

#include <vector>
#include <d3d11.h>
#include "D3D11_Core.h"
#include "Shaders/VertexShader.h"
#include "Shaders/InputLayout.h"
#include "Shaders/PixelShader.h"
#include "Texture.h"

/// <summary>
/// Takes care loading and unloading the resources
/// </summary>
/// <remarks>
/// For now this is a naive implementation that performs the loading operation
/// each time it is requested
/// </remarks>
class ContentManager
{
  public:
    /// <summary>
    /// Creates a content manager
    /// </summary>
    /// <param name="d3d11">
    /// D3D11 data needed to load content
    /// </param>
    ContentManager(D3D11_Core& d3d11);
    
    /// <summary>
    /// Cleans up the content manager
    /// </summary>
    ~ContentManager();
    
    /// <summary>
    /// Loads a vertex shader
    /// </summary>
    /// <param name="file">
    /// path to the file that contains the already compiled shader code
    /// </param>
    /// <param name="layout">
    /// layout of inputs to the vertex shader
    /// </param>
    /// <param name="buffer_widths">
    /// number of bytes in each constant buffer to create
    /// </param>
    /// <returns>
    /// pointer to the loaded shader
    /// NULL if an error occurred
    /// </returns>
    VertexShader* LoadVertexShader(const WCHAR* file,
      const InputLayout& layout,const std::vector<UINT>& buffer_widths);
    
    /// <summary>
    /// Loads a pixel shader
    /// </summary>
    /// <param name="file">
    /// path to the file that contains the already compiled shader code
    /// </param>
    /// <param name="buffer_widths">
    /// number of bytes in each constant buffer to create
    /// </param>
    /// <returns>
    /// pointer to the loaded shader
    /// NULL if an error occurred
    /// </returns>
    PixelShader* LoadPixelShader(const WCHAR* file,
      const std::vector<UINT>& buffer_widths);
    
    /// <summary>
    /// Loads a texture (must be a dds file or a format supported by WIC)
    /// </summary>
    /// <param name="file">
    /// path to the file that contains the texture
    /// </param>
    /// <returns>
    /// pointer to the loaded texture
    /// NULL if an error occurred
    /// </returns>
    Texture* LoadTexture(const WCHAR* file);
    
    /// <summary>
    /// Informs the content manager that the shader can be unloaded
    /// </summary>
    /// <param name="shader">
    /// shader that can be unloaded
    /// </param>
    void UnloadShader(VertexShader* shader);
    
    /// <summary>
    /// Informs the content manager that the shader can be unloaded
    /// </summary>
    /// <param name="shader">
    /// shader that can be unloaded
    /// </param>
    void UnloadShader(PixelShader* shader);
    
    /// <summary>
    /// Informs the content manager that the texture can be unloaded
    /// </summary>
    /// <param name="tex">
    /// texture that can be unloaded
    /// </param>
    void UnloadTexture(Texture* tex);
    
  private:
    // disabled
    ContentManager(const ContentManager& cpy);
    ContentManager& operator=(const ContentManager& cpy);
    
    /// <summary>
    /// Loads the entire contents of a binary file into memory
    /// <remarks>
    /// This only handles files whose length in bytes fits into a UINT and will
    /// fail to load longer files.  This shouldn't be too sever a limitation
    /// and if needed, can be removed by changing length to a larger type and
    /// ensuring ifstream can also handle larger file sizes as well.
    /// </remarks>
    /// </summary>
    /// <param name="file">
    /// path to the file to load
    /// </param>
    /// <param name="data">
    /// where to put the loaded data
    /// <remarks>
    /// This function will dynamically allocate an exact fit buffer for the
    /// file contents via new[] and the caller is responsible for freeing it.
    /// Also, if the function returns false, the memory is freed by this
    /// function.
    /// </remarks>
    /// </param>
    /// <param name="length">
    /// number of bytes read
    /// </param>
    /// <returns>
    /// true if the file was successfully loaded
    // false otherwise
    /// </returns>
    bool LoadFile(const WCHAR* file,char*& data,UINT& length);
    
    /// <summary>
    /// Creates constant buffers for a shader
    /// </summary>
    /// <param name="buffer_widths">
    /// number of bytes in each constant buffer to create
    /// </param>
    /// <param name="buffers">
    /// where to put the created buffers
    /// </param>
    /// <param name="file">
    /// file name of the shader the constant buffers are being created for.
    /// This is just for log messages, so it only needs to be non-NULL if
    /// log_would_output() returns true.  Otherwise this argument is ignored.
    /// </param>
    /// <returns>
    /// true if all buffers were successfully created
    /// false if any buffer failed to be created (in this case all buffers in
    /// the buffers argument are not valid and should be ignored)
    /// </returns>
    bool CreateBuffers(const std::vector<UINT>& buffer_widths,
      std::vector<ID3D11Buffer*>& buffers,const WCHAR* file);
    
    /// <summary>
    /// D3D11 data needed to load content
    /// </summary>
    D3D11_Core& m_d3d11;
};

#endif /* CONTENTMANAGER_H */
