#ifndef D3D11_CORE_H
#define D3D11_CORE_H

#include <d3d11.h>

/// <summary>
/// Manages the core needed variables to use D3D11
/// </summary>
class D3D11_Core
{
  public:
    /// <summary>
    /// Setups D3D11 for the specified window
    /// </summary>
    /// <param name="wnd">
    /// handle to the window to setup D3D11 for
    /// </param>
    /// <returns>
    /// pointer to the D3D11_Core instance for the window on success
    /// NULL on error
    /// </returns>
    static D3D11_Core* Create(HWND& wnd);
    
    /// <summary>
    /// Cleans up the D3D11 core.  All resources created with this D3D11 core
    /// instance must be cleaned up before calling this.
    /// </summary>
    ~D3D11_Core();
    
#ifndef PUBLIC_HEADER
    /// <summary>
    /// Switches the graphics side of things from fullscreen to windowed and
    /// vice versa
    /// <summary>
    /// <param name="width">
    /// width of the client area, in pixels 
    /// </param>
    /// <param name="height">
    /// height of the client area, in pixels
    /// </param>
    /// <param name="enable">
    /// true  if the window should be fullscreen
    /// false if the window should be in windowed mode
    /// </param>
    void Fullscreen(UINT width,UINT height,bool enable);
    
    /// <summary>
    /// Handles updating the D3D11 when the window is resized
    /// </summary>
    /// <param name="width">
    /// width of the new client area, in pixels 
    /// </param>
    /// <param name="height">
    /// height of the new client area, in pixels
    /// </param>
    void OnResize(UINT width,UINT height);
    
    /// <summary>
    /// Retrieves the D3D11 device
    /// </summary>
    /// <returns>
    /// D3D11 device
    /// </returns>
    ID3D11Device* GetDevice();
#endif /* PUBLIC_HEADER */
    
    /// <summary>
    /// Retrieves the D3D11 immediate context
    /// </summary>
    /// <returns>
    /// D3D11 immediate context
    /// </returns>
    ID3D11DeviceContext* GetImmediateContext();
    
    /// <summary>
    /// Retrieves a viewport specification that covers the entire window client
    /// area (which is automatically updated whenever the window is resized)
    /// </summary>
    /// <returns>
    /// viewport specification that covers the window's client area
    /// </returns>
    const D3D11_VIEWPORT& GetDefaultViewport() const;
    
    /// <sumamry>
    /// Clears the back buffer used for rendering to black
    /// </sumamry>
    void ClearBackBuffer();
    
    /// <sumamry>
    /// Clears the depth buffer used for rendering
    /// </sumamry>
    void ClearDepthBuffer();
    
    /// <sumamry>
    /// Draws index primitives to the specified context
    /// </sumamry>
    /// <param name="context">
    /// where to draw to
    /// </param>
    /// <param name="primitive">
    /// which primitive to draw
    /// </param>
    /// <param name="start_index">
    /// start index into the context's index buffer
    /// </param>
    /// <param name="num_indices">
    /// number of indices in the index buffer for the context to use for
    /// drawing
    /// </param>
    void DrawIndexedPrimitives(ID3D11DeviceContext* context,
      D3D11_PRIMITIVE_TOPOLOGY primitive,UINT start_index,UINT num_indices);
    
    /// <sumamry>
    /// Draws instanced primitives to the specified context
    /// <remarks>
    /// The instance buffer needs to be bound before calling this function
    /// </remarks>
    /// </sumamry>
    /// <param name="context">
    /// where to draw to
    /// </param>
    /// <param name="primitive">
    /// which primitive to draw
    /// </param>
    /// <param name="start_index">
    /// start index into the context's index buffer
    /// </param>
    /// <param name="num_indices">
    /// number of indices in the index buffer for the context to use for
    /// drawing
    /// </param>
    /// <param name="start_instance">
    /// which instance to start drawing with
    /// </param>
    /// <param name="num_instances">
    /// number of indices to draw
    /// </param>
    void DrawInstancedPrimitives(ID3D11DeviceContext* context,
      D3D11_PRIMITIVE_TOPOLOGY primitive,UINT start_index,UINT num_indices,
      UINT start_instance,UINT num_instances);
    
    /// <summary>
    /// Swaps the back and front buffers to display the frame to the user
    /// </summary>
    void Swap();
    
  private:
    D3D11_Core(ID3D11Device* device,ID3D11DeviceContext* immediate_context,
      IDXGISwapChain* swap_chain,ID3D11RenderTargetView* render_target_view,
      ID3D11Texture2D* depth_stencil,
      ID3D11DepthStencilView* depth_stencil_view,
      D3D11_VIEWPORT viewport);
    
    // disabled
    D3D11_Core(const D3D11_Core& cpy);
    D3D11_Core& operator=(const D3D11_Core& cpy);
    
    /// <summary>
    /// D3D11 device
    /// </summary>
    ID3D11Device*           m_device;
    
    /// <summary>
    /// immediate context
    /// </summary>
    ID3D11DeviceContext*    m_immediate_context;
    
    /// <summary>
    /// swap chain
    /// </summary>
    IDXGISwapChain*         m_swap_chain;
    
    /// <summary>
    /// render target view
    /// </summary>
    ID3D11RenderTargetView* m_render_target_view;
    
    /// <summary>
    /// storage for the actual depth stencil
    /// </summary>
    ID3D11Texture2D*        m_depth_stencil;
    
    /// <summary>
    /// view for the depth stencil
    /// </summary>
    ID3D11DepthStencilView* m_depth_stencil_view;
    
    /// <summary>
    /// viewport specification that covers the window's client area
    /// </summary>
    D3D11_VIEWPORT          m_default_viewport;
};

#endif /* D3D11_CORE_H */
