#ifndef RENDER_PIPELINE_H
#define RENDER_PIPELINE_H

#include "Shaders/VertexShader.h"
#include "Shaders/PixelShader.h"
#include "BlendState.h"
#include "DepthStencilState.h"

class ContentManager;

/// <summary>
/// Container for all the various stages of the rendering process
/// </summary>
class RenderPipeline
{
  public:
    /// <summary>
    /// Creates a new rendering pipeline instance with the specified stages.
    /// All others will be disabled during rendering
    /// </summary>
    /// <param name="vertex">
    /// vertex shader to use
    /// <remarks>
    /// This must point to a valid vertex shader (i.e. not NULL) and it is up
    /// to the caller to ensure this
    /// </remarks>
    /// </param>
    /// <param name="pixel">
    /// pixel shader to use
    /// <remarks>
    /// This must point to a valid vertex shader (i.e. not NULL) and it is up
    /// to the caller to ensure this
    /// </remarks>
    /// </param>
    /// <param name="blend">
    /// blend state to use
    /// <remarks>
    /// Can be NULL which will cause D3D11's default to be used
    /// </remarks>
    /// </param>
    /// <param name="depth">
    /// depth stencil state to use
    /// <remarks>
    /// Can be NULL which will cause D3D11's default to be used
    /// </remarks>
    /// </param>
    RenderPipeline(VertexShader* vertex,PixelShader* pixel,
      BlendState* blend = NULL,DepthStencilState* depth = NULL);
    
    /// <summary>
    /// Makes the entire pipeline active so it can be used for rendering.
    /// <summary>
    /// <remarks>
    /// Any updating of the resources of the various stages should be done
    /// before calling this function.
    /// </remarks>
    /// <param name="context">
    /// D3D11 context to apply the pipeline to
    /// </param>
    void MakeActive(ID3D11DeviceContext* context);
    
    /// <summary>
    /// Retrieves the vertex shader
    /// </summary>
    /// <remarks>
    /// The shader is returned in mutable (non-const) form so that is resources
    /// can be updated
    /// </remarks>
    /// <returns>
    /// pointer to the vertex shader
    /// </returns>
    VertexShader* GetVertexShader() const;
    
    /// <summary>
    /// Retrieves the pixel shader
    /// </summary>
    /// <remarks>
    /// The shader is returned in mutable (non-const) form so that is resources
    /// can be updated
    /// </remarks>
    /// <returns>
    /// pointer to the vertex shader
    /// </returns>
    PixelShader* GetPixelShader() const;
    
  private:
    // disabled
    RenderPipeline();
    RenderPipeline(const RenderPipeline& cpy);
    RenderPipeline& operator=(const RenderPipeline& cpy);
    
    /// <summary>
    /// Cleans up the pipeline
    /// </summary>
    /// <remarks>
    /// private so that ContentManager has to be used to clean it up
    /// </remarks>
    ~RenderPipeline();
    
    // so a ContentManager instance can deallocate at the appropriate time
    friend ContentManager;
    
    /// <summary>
    /// vertex shader stage
    /// </summary>
    VertexShader* m_vertex;
    
    /// <summary>
    /// pixel shader stage
    /// </summary>
    PixelShader* m_pixel;
    
    /// <summary>
    /// blend state
    /// </summary>
    BlendState* m_blend;
    
    /// <summary>
    /// depth stencil state
    /// </summary>
    DepthStencilState* m_depth;
};

#endif /* RENDER_PIPELINE_H */
