#ifndef INPUT_LAYOUT_H
#define INPUT_LAYOUT_H

#include <d3d11.h>
#include "Semantics.h"

/// <summary>
/// Describes the input layout for a vertex shader
/// </summary>
/// <remarks>
/// Due to each element being able to be set only once,
/// ContentManger::LoadVertexShader could cache the graphics library
/// representation in the InputLayout instance.  But since
/// ID3D11Device::CreateInputLayout takes the shader byte code (documented as
/// only for validation, but erring on the side of caution here), that is not
/// done in case the layout is used with multiple shaders.
/// </remarks>
class InputLayout
{
  public:
    /// <summary>
    /// An empty layout with the specified number of elements
    /// </summary>
    /// <param name="num">
    /// number of elements that make up the layout
    /// </param>
    /// <remarks>
    /// Each element must be set via SetNextElement before passing the instance
    /// to a ContentManager
    /// </remarks>
    InputLayout(UINT num);
    
    ~InputLayout();
    
    /// <summary>
    /// Sets the input element description of the next entry in the layout
    /// </summary>
    /// <remarks>
    /// Requires:
    /// 1) non-instance data is in the first input slot/vertex buffer, and
    ///    instance data is in the second.
    /// 2) per vertex data to be tightly packed, starting at offset 0, and
    ///    in the order they are added to the layout instance
    /// 3) per instance data to be tightly packed, starting at offset 0, and
    ///    in the order they are added to the layout instance
    /// </remarks>
    /// <param name="semantic">
    /// which semantic to use for the entry
    /// </remarks>
    /// </param>
    /// <param name="index">
    /// semantic index
    /// </param>
    /// <param name="format">
    /// format of the entry
    /// <remarks>
    /// Looked into alternatives to taking a DXGI_FORMAT here, but the enum has
    /// so many different types that can be used as input to a vertex shader
    /// that it is impractical to do anything besides take it directly for now
    /// </remarks>
    /// </param>
    /// <param name="instance">
    /// true if this is for per instance data
    /// false for per vertex
    /// </param>
    /// <returns>
    /// true if the element was successfully set
    /// false otherwise (i.e. no element was available to be set)
    /// </returns>
    bool SetNextElement(Semantics semantic,UINT index,
      DXGI_FORMAT format,bool instance);
    
    /// <summary>
    /// Retrieves the number of elements in the layout
    /// </summary>
    /// <returns>
    /// number of elements
    /// </returns>
    UINT GetNum() const;
    
#ifndef PUBLIC_HEADER
    /// <summary>
    /// Retrieves the D3D11 layout description
    /// </summary>
    /// <returns>
    /// D3D11 layout
    /// </returns>
    const D3D11_INPUT_ELEMENT_DESC* GetLayout() const;
#endif /* PUBLIC_HEADER */
    
  private:
    // disabled
    InputLayout();
    InputLayout(const InputLayout& cpy);
    InputLayout& operator=(const InputLayout& cpy);
    
    /// <summary>
    /// array of input element descriptions that make up the layout
    /// </summary>
    D3D11_INPUT_ELEMENT_DESC* m_layout;
    
    /// <summary>
    /// number of entries in m_layout
    /// </summary>
    UINT m_num;
    
    /// <summary>
    /// index into m_layout of the next element to set
    /// </summary>
    UINT m_next;
};

#endif /* INPUT_LAYOUT_H */
