#ifndef PIXEL_SHADER_H
#define PIXEL_SHADER_H

#include <vector>
#include "ShaderBase.h"
#include "Graphics/D3D11_Core.h"
#include "Graphics/Texture.h"
#include "Graphics/TextureSampler.h"

class ContentManager;

/// <summary>
/// Pixel/fragment shader
/// </summary>
class PixelShader : public ShaderBase
{
  public:
    /// <summary>
    /// Creates a pixel shader
    /// </summary>
    /// <param name="pixel">
    /// pixel shader
    /// </param>
    /// <param name="buffers">
    /// constant buffers for the shader
    /// </param>
    PixelShader(ID3D11PixelShader* pixel,
      const std::vector<ID3D11Buffer*>& buffers);
    
    /// <summary>
    /// Setups the shader to be used for rendering
    /// <summary>
    /// <param name="context">
    /// D3D11 context to apply the shader to
    /// </param>
    void MakeActive(ID3D11DeviceContext* context);
    
    /// <summary>
    /// Updates the rendering context with the latest values of the constant
    /// buffers (useful for updating constant buffers after the call to
    /// MakeActive)
    /// <summary>
    /// <param name="context">
    /// D3D11 context to update the constant buffers of
    /// </param>
    void UpdateConstantBuffers(ID3D11DeviceContext* context);
    
    /// <summary>
    /// Supplies the specified texture to the pixel shader, which will be used
    /// during MakeActive until the value is changed by another call to this
    /// function with the same index
    /// </summary>
    /// <param name="tex">
    /// texture to supply to the pixel shader
    /// </param>
    /// <param name="index">
    /// which texture index in the shader the texture should use
    /// </param>
    void UseTexture(Texture* tex,UINT index);
    
    /// <summary>
    /// Supplies the specified texture sampler to the pixel shader, which will
    /// be used during MakeActive until the value is changed by another call to
    /// this function with the same index
    /// </summary>
    /// <param name="context">
    /// d3d11 context to supply the texture sampler to
    /// </param>
    /// <param name="sampler">
    /// texture sampler to supply to the pixel shader
    /// </param>
    /// <param name="index">
    /// which texture index in the shader the texture sampler should be
    /// supplied to
    /// </param>
    void UseTextureSampler(TextureSampler* sampler,UINT index);
    
  protected:
    /// <summary>
    /// Cleans up the shader
    /// </summary>
    /// <remarks>
    /// protected so that ContentManager has to be used to clean it up
    /// </remarks>
    ~PixelShader();
    
  private:
    // disabled
    PixelShader(const PixelShader& cpy);
    PixelShader& operator=(const PixelShader& cpy);
    
    // so a ContentManager instance can deallocate at the appropriate time
    friend ContentManager;
    
    /// <summary>
    /// pixel shader
    /// </summary>
    ID3D11PixelShader* m_pixel;
    
    /// <summary>
    /// textures for the shader
    /// </summary>
    ID3D11ShaderResourceView*
      m_textures[D3D11_COMMONSHADER_INPUT_RESOURCE_SLOT_COUNT];
    
    /// <summary>
    /// texture samplers for the shader
    /// </summary>
    ID3D11SamplerState* m_samplers[D3D11_COMMONSHADER_SAMPLER_SLOT_COUNT];
};

#endif /* PIXEL_SHADER_H */
