#ifndef SHADER_BASE_H
#define SHADER_BASE_H

#include <vector>
#include <d3d11.h>

/// <summary>
/// Base class for the various shaders, which represent programmable stages in
/// the rendering pipeline
/// </summary>
class ShaderBase
{
  public:
    ShaderBase(const std::vector<ID3D11Buffer*>& buffers);
    
    /// <summary>
    /// Sets the value of the desired constant buffer
    /// <remarks>
    /// The value of the constant buffer will only be applied to a rendering
    /// context on the next call to MakeActive or UpdateConstantBuffers
    /// </remarks>
    /// <summary>
    /// <param name="context">
    /// d3d11 context to update the value of the buffer in
    /// </param>
    /// <param name="id">
    /// 0 based index of which constant buffer to set the value of
    /// </param>
    /// <param name="val">
    /// pointer to the data to update the buffer with.  The data must be the
    /// same size as was used to create the buffer
    /// <remarks>
    /// any matrices that are pointed to by this need to be transposed before
    /// calling this
    /// </remarks>
    /// </param>
    void SetConstantBuffer(ID3D11DeviceContext* context,UINT id,
      const void* val);
    
    /// <summary>
    /// Setups the shader to be used for rendering
    /// <summary>
    /// <param name="context">
    /// D3D11 context to apply the shader to
    /// </param>
    virtual void MakeActive(ID3D11DeviceContext* context) = 0;
    
    /// <summary>
    /// Updates the rendering context with the latest values of the constant
    /// buffers (useful for updating constant buffers after the call to
    /// MakeActive)
    /// <summary>
    /// <param name="context">
    /// D3D11 context to update the constant buffers of
    /// </param>
    virtual void UpdateConstantBuffers(ID3D11DeviceContext* context) = 0;
    
  protected:
    /// <summary>
    /// Cleans up the shader
    /// </summary>
    /// <remarks>
    /// protected so that ContentManager has to be used to clean it up and so
    /// subclasses still have access to it
    /// </remarks>
    virtual ~ShaderBase();
    
    /// <summary>
    /// Retieves the constant buffer array, which is
    /// D3D11_COMMONSHADER_CONSTANT_BUFFER_API_SLOT_COUNT elements long
    /// </summary>
    /// <remarks>
    /// This is a get function instead of applying the buffers to the pipeline
    /// since each stage has its own function for setting constant buffers and
    /// the subclasses known which stage they are working on
    /// </remarks>
    /// <returns>
    /// constant buffer array
    /// </returns>
    ID3D11Buffer*const* GetBuffers() const;
    
  private:
    // disabled
    ShaderBase();
    ShaderBase(const ShaderBase& cpy);
    ShaderBase& operator=(const ShaderBase& cpy);
    
    /// <summary>
    /// constant buffers for the shader
    /// </summary>
    ID3D11Buffer* m_buffers[D3D11_COMMONSHADER_CONSTANT_BUFFER_API_SLOT_COUNT];
};

#endif /* SHADER_BASE_H */
