#ifndef VERTEX_SHADER_H
#define VERTEX_SHADER_H

#include <vector>
#include "ShaderBase.h"
#include "Graphics/D3D11_Core.h"

class ContentManager;

/// <summary>
/// Vertex shader
/// </summary>
/// <remarks>
/// In D3D11, also takes care of the input assembler stage
/// </remarks>
class VertexShader : public ShaderBase
{
  public:
    /// <summary>
    /// Creates a vertex shader
    /// </summary>
    /// <param name="vertex">
    /// vertex shader
    /// </param>
    /// <param name="layout">
    /// layout for vertex shader input
    /// </param>
    /// <param name="buffers">
    /// constant buffers
    /// </param>
    VertexShader(ID3D11VertexShader* vertex,ID3D11InputLayout* layout,
      const std::vector<ID3D11Buffer*>& buffers);
    
    /// <summary>
    /// Setups the vertex shader to be used for rendering
    /// <summary>
    /// <param name="context">
    /// D3D11 context to apply the shader to
    /// </param>
    void MakeActive(ID3D11DeviceContext* context);
    
    /// <summary>
    /// Updates the rendering context with the latest values of the constant
    /// buffers (useful for updating constant buffers after the call to
    /// MakeActive)
    /// <summary>
    /// <param name="context">
    /// D3D11 context to update the constant buffers of
    /// </param>
    void UpdateConstantBuffers(ID3D11DeviceContext* context);
    
  protected:
    /// <summary>
    /// Cleans up the shader
    /// </summary>
    /// <remarks>
    /// protected so that ContentManager has to be used to clean it up
    /// </remarks>
    ~VertexShader();
    
  private:
    // disabled
    VertexShader(const VertexShader& cpy);
    VertexShader& operator=(const VertexShader& cpy);
    
    // so a ContentManager instance can deallocate at the appropriate time
    friend ContentManager;
    
    /// <summary>
    /// vertex shader
    /// </summary>
    ID3D11VertexShader* m_vertex;
    
    /// <summary>
    /// layout for vertex shader input
    /// </summary>
    ID3D11InputLayout* m_vertex_layout;
};

#endif /* VERTEX_SHADER_H */
