#ifndef VIEWPORTS_H
#define VIEWPORTS_H

#include <d3d11.h>

/// <summary>
/// Takes care of managing viewports
/// </summary>
/// <remarks>
/// Instances of this class manage multiple viewports instead of a single one
/// due to ID3D11DeviceContext::RSSetViewport taking a number of viewports and
/// a pointer to an array with the specified number of viewports.  It doesn't
/// provide a way to set a single arbitrary viewport at a time (such as
/// IASetVertexBuffers taking the start slot as an argument)
/// </remarks>
class Viewports
{
  public:
    Viewports();
    ~Viewports();
    
    /// <summary>
    /// Setups the viewports to be used for rendering
    /// <summary>
    /// <param name="context">
    /// D3D11 context to apply the viewports to
    /// </param>
    void MakeActive(ID3D11DeviceContext* context);
    
    /// <summary>
    /// Setups the viewports to be used for rendering
    /// <remarks>
    /// This overload of MakeActive is for convience/maximizing allocated space
    /// usage.  It allows for a program to create 1 instance of Viewports and
    /// use all of its viewports (space for which is allocated with the
    /// instance) while only applying a continuous seletion of them at a time
    /// (i.e. a 4 way split screen game could use 4 viewports in a single
    /// Viewports instance and apply the viewports to the rendering pipeline
    /// individually).
    /// </remarks>
    /// <summary>
    /// <param name="context">
    /// D3D11 context to apply the viewports to
    /// </param>
    /// <param name="start">
    /// index of the first viewport to apply to the rendering pipeline
    /// </param>
    /// <param name="num">
    /// number of viewports to apply
    /// </param>
    /// <return>
    /// true if the viewports were applied
    /// false otherwise (i.e. start >= GetNumInUse() or
    ///   (start + num) >= GetNumInUse())
    /// </return>
    bool MakeActive(ID3D11DeviceContext* context,UINT start,UINT num);
    
    /// <summary>
    /// Sets the next available viewport to the specified value, and increases
    /// the number of viewports in use
    /// </summary>
    /// <param name="viewport">
    /// settings for the new viewport
    /// </param>
    /// <returns>
    /// true  if the viewport was added successfully
    /// false otherwise (ie trying to add too many viewports)
    /// </returns>
    bool SetNextViewport(const D3D11_VIEWPORT& viewport);
    
    /// <summary>
    /// Sets the specified viewport to the specified value
    /// </summary>
    /// <param name="index">
    /// index of the viewport to set
    /// </param>
    /// <param name="viewport">
    /// settings for the viewport
    /// </param>
    /// <returns>
    /// true  if the viewport was updated successfully
    /// false otherwise (ie trying to set an unused viewport)
    /// </returns>
    bool SetViewport(UINT index,const D3D11_VIEWPORT& viewport);
    
    /// <summary>
    /// Retrieves the requested viewport's data
    /// </summary>
    /// <param name="index">
    /// index of the viewport to retrieve
    /// </param>
    /// <param name="viewport">
    /// where to put the viewport's data
    /// </param>
    /// <returns>
    /// true if the viewport could be retrieved
    /// false otherwise
    /// </returns>
    bool GetViewport(UINT index,D3D11_VIEWPORT& viewport) const;
    
    /// <summary>
    /// Retrieves the number of viewports currently in use
    /// </summary>
    /// <returns>
    /// number of viewports in use
    /// </returns>
    UINT GetNumInUse() const;
    
    /// <summary>
    /// Retrieves the maximium number of viewports that can be used by a single
    /// instance
    /// </summary>
    /// <returns>
    /// maximium number of viewports that can be used
    /// </returns>
    static UINT GetMaxViewports();
    
  private:
    // disabled
    Viewports(const Viewports& cpy);
    Viewports& operator=(const Viewports& cpy);
    
    /// <summary>
    /// array of the various viewports
    /// </summary>
    D3D11_VIEWPORT m_viewports
      [D3D11_VIEWPORT_AND_SCISSORRECT_OBJECT_COUNT_PER_PIPELINE];
    
    /// <summary>
    /// index into m_viewports for which viewport is the next one to use
    /// </summary>
    UINT m_next;
};

#endif /* VIEWPORTS_H */
