#include <fstream>
#include <sstream>
#include "Graphics/ContentManager.h"
#include "log.h"
#include "DDSTextureLoader.h"
#include "WICTextureLoader.h"
using namespace std;
using namespace DirectX;

ContentManager::ContentManager(D3D11_Core& d3d11)
:m_d3d11(d3d11)
{
}

ContentManager::~ContentManager()
{
}

VertexShader* ContentManager::LoadVertexShader(const WCHAR* file,
  const InputLayout& layout,const vector<UINT>& buffer_widths)
{
  ID3D11Device* device = m_d3d11.GetDevice();
  
  // load the compiled shader code
  char* data = NULL;
  UINT length = 0;
  if (!LoadFile(file,data,length))
  {
    if (log_would_output())
    {
      wostringstream buffer;
      buffer << L"Failed to load \"" << file << L"\"\n";
      log_print(buffer.str().c_str());
    }
    return NULL;
  }
  
  // create the vertex shader
  ID3D11VertexShader* vertex_shader = NULL;
  if (FAILED(device->CreateVertexShader(data,length,NULL,&vertex_shader)))
  {
    if (log_would_output())
    {
      wostringstream buffer;
      buffer << L"Failed to create the vertex shader for \"" << file
             << L"\"\n";
      log_print(buffer.str().c_str());
    }
    delete[]data;
    return NULL;
  }
  
  // create the input layout
  ID3D11InputLayout* vertex_layout = NULL;
  if (FAILED(device->CreateInputLayout(layout.GetLayout(),layout.GetNum(),
    data,length,&vertex_layout)))
  {
    if (log_would_output())
    {
      wostringstream buffer;
      buffer << L"Failed to create the input layout for \"" << file
             << L"\"\n";
      log_print(buffer.str().c_str());
    }
    vertex_shader->Release();
    delete[]data;
    return NULL;
  }
  delete[]data;
  
  // create the constant buffers
  vector<ID3D11Buffer*> buffers;
  if (!CreateBuffers(buffer_widths,buffers,file))
  {
    vertex_shader->Release();
    vertex_layout->Release();
    return NULL;
  }
  return new VertexShader(vertex_shader,vertex_layout,buffers);
}

PixelShader* ContentManager::LoadPixelShader(const WCHAR* file,
  const vector<UINT>& buffer_widths)
{
  ID3D11Device* device = m_d3d11.GetDevice();
  
  // load the compiled shader code
  char* data = NULL;
  UINT length = 0;
  if (!LoadFile(file,data,length))
  {
    if (log_would_output())
    {
      wostringstream buffer;
      buffer << L"Failed to load \"" << file << L"\"\n";
      log_print(buffer.str().c_str());
    }
    return NULL;
  }
  
  // create the pixel shader
  ID3D11PixelShader* pixel_shader = NULL;
  if (FAILED(device->CreatePixelShader(data,length,NULL,&pixel_shader)))
  {
    if (log_would_output())
    {
      wostringstream buffer;
      buffer << L"Failed to create the pixel shader for \"" << file
             << L"\"\n";
      log_print(buffer.str().c_str());
    }
    delete[]data;
    return NULL;
  }
  delete[]data;
  
  // create the constant buffers
  vector<ID3D11Buffer*> buffers;
  if (!CreateBuffers(buffer_widths,buffers,file))
  {
    pixel_shader->Release();
    return NULL;
  }
  return new PixelShader(pixel_shader,buffers);
}

Texture* ContentManager::LoadTexture(const WCHAR* file)
{
  ID3D11ShaderResourceView* tex = NULL;
  if (wstring(file).rfind(L".dds") != wstring::npos)
  {
    // use DirectXTK dds loader
    if (FAILED(CreateDDSTextureFromFile(m_d3d11.GetDevice(),file,NULL,&tex)))
    {
      if (log_would_output())
      {
        wostringstream buffer;
        buffer << L"Failed to load texture \"" << file << L"\"\n";
        log_print(buffer.str().c_str());
        return NULL;
      }
    }
  }
  else
  {
    // use DirectXTK WIC loader
    if (FAILED(CreateWICTextureFromFile(m_d3d11.GetDevice(),
      m_d3d11.GetImmediateContext(),file,NULL,&tex)))
    {
      if (log_would_output())
      {
        wostringstream buffer;
        buffer << L"Failed to load texture \"" << file << L"\"\n";
        log_print(buffer.str().c_str());
        return NULL;
      }
    }
  }
  return new Texture(tex);
}

void ContentManager::UnloadShader(VertexShader* shader)
{
  delete shader;
}

void ContentManager::UnloadShader(PixelShader* shader)
{
  delete shader;
}

void ContentManager::UnloadTexture(Texture* tex)
{
  delete tex;
}

bool ContentManager::LoadFile(const WCHAR* file,char*& data,UINT& length)
{
  ifstream in(file,ios_base::in|ios_base::binary);
  if (!in.good())
  {
    return false;
  }
  
  // get the file length, in bytes
  in.seekg(0,ios_base::end);
  streampos len = in.tellg();
  if (len > UINT_MAX)
  {
    // only handles files whose length fits into a UINT for now
    in.close();
    return false;
  }
  length = (UINT)len;
  
  // load the file
  in.seekg(0,ios_base::beg);
  data = new char[length];
  if (!in.read(data,length))
  {
    delete[]data;
    in.close();
    return false;
  }
  
  in.close();
  return true;
}

bool ContentManager::CreateBuffers(const vector<UINT>& buffer_widths,
  vector<ID3D11Buffer*>& buffers,const WCHAR* file)
{
  ID3D11Device* device = m_d3d11.GetDevice();
  
  // setup common values for constant buffers
  D3D11_BUFFER_DESC bd;
  ZeroMemory(&bd,sizeof(bd));
  bd.Usage          = D3D11_USAGE_DEFAULT;
  bd.BindFlags      = D3D11_BIND_CONSTANT_BUFFER;
  bd.CPUAccessFlags = 0;
  
  // query for each buffer size and create it
  buffers.reserve(buffer_widths.size());
  vector<UINT>::const_iterator width_it = buffer_widths.begin();
  int i = 0;
  while (width_it != buffer_widths.end())
  {
    // create the buffer
    ID3D11Buffer* buffer = NULL;
    bd.ByteWidth = *width_it;
    if (SUCCEEDED(device->CreateBuffer(&bd,NULL,&buffer)))
    {
      buffers.push_back(buffer);
    }
    else
    {
      if (log_would_output())
      {
        wostringstream buffer;
        buffer << L"Failed to create the constant buffer " << i << L"for \""
               << file << L"\"\n";
        log_print(buffer.str().c_str());
      }
      
      // clean up the ones that went through already
      vector<ID3D11Buffer*>::iterator buff_it = buffers.begin();
      while (buff_it != buffers.end())
      {
        (*buff_it)->Release();
        
        ++buff_it;
      }
      return false;
    }
    
    ++width_it;
    ++i;
  }
  
  return true;
}
