#include <sstream>
#include "Game.h"
#include "Time/Timer.h"
#include "log.h"
using namespace std;

Game::Game(WCHAR* title)
:m_mspf(1000 / 60),
 m_run(true)
{
  // create the game window
  m_window = Window::CreateDefaultWindow(title,ResizeHandler,this);
  
  // handle the error case of failing to create the window
  if (!m_window)
  {
    exit(1);
  }
  
  // create the D3D11 core for the window
  m_graphics = D3D11_Core::Create(m_window->GetHandle());
}

Game::~Game()
{
  delete m_graphics;
  delete m_window;
}

void Game::Run()
{
  // perform init
  m_content = new ContentManager(*m_graphics);
  LoadContent();
  
  // get a timer
  Timer* timer = Timer::GetTimer();
  
  // message loop
  MSG msg = {0};
  while(m_run && msg.message != WM_QUIT)
  {
    // handle messages
    if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
    {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }
    // check if it's time for another update/render cycle
    else if (timer->CheckDelta(m_mspf))
    {
      Update(m_mspf);
      Render(m_mspf);
    }
  }
  
  // perform cleanup
  UnloadContent();
  delete m_content;
}

void Game::Resize(UINT width,UINT height)
{
  m_window->Resize(width,height);
}

void Game::Fullscreen(bool enable)
{
  UINT width;
  UINT height;
  GetWindowClientDim(width,height);
  
  m_graphics->Fullscreen(width,height,enable);
}

bool Game::GetDesktopSize(UINT& width,UINT& height) const
{
  width = GetSystemMetrics(SM_CXSCREEN);
  if (width == 0)
  {
    if (log_would_output())
    {
      ostringstream out;
      out << "Failed to get desktop width, error code " << GetLastError()
          << '\n';
      log_print(out.str().c_str());
    }
    return false;
  }
  
  height = GetSystemMetrics(SM_CYSCREEN);
  if (height == 0)
  {
    if (log_would_output())
    {
      ostringstream out;
      out << "Failed to get desktop height, error code " << GetLastError()
          << '\n';
      log_print(out.str().c_str());
    }
    return false;
  }
  
  return true;
}

void Game::Exit()
{
  m_run = false;
}

void Game::GetWindowClientDim(UINT& width,UINT& height) const
{
  RECT rect;
  GetClientRect(m_window->GetHandle(),&rect);
  width  = rect.right  - rect.left;
  height = rect.bottom - rect.top;
}

const MouseState& Game::GetMouseState() const
{
  return m_window->GetMouseState();
}

const KeyboardState& Game::GetKeyboardState() const
{
  return m_window->GetKeyboardState();
}

void Game::OnResize(UINT width,UINT height)
{
  m_graphics->OnResize(width,height);
}

D3D11_Core& Game::GetGraphics()
{
  return *m_graphics;
}

ContentManager& Game::GetContentManager()
{
  return *m_content;
}

UINT Game::GetMSPerFrame() const
{
  return m_mspf;
}

void Game::SetMSPerFrame(UINT mspf)
{
  m_mspf = mspf;
}

void Game::ResizeHandler(void* arg,UINT width,UINT height)
{
  Game* game = (Game*)arg;
  game->OnResize(width,height);
}

void Game::Render(UINT ms)
{
  m_graphics->ClearBackBuffer();
  m_graphics->ClearDepthBuffer();
  
  Draw(ms);
  
  m_graphics->Swap();
}
